import { node_path as path } from '../runtime/mod.ts';

// ----------------------------------------------------------------------------
// File-related types
// ----------------------------------------------------------------------------

export enum FileLang {
	TypeScript = 'TypeScript',
	JavaScript = 'JavaScript',
	CSS = 'CSS',
	Plain = 'Plain Text',
}

export interface HighlightingOptions {
	numbers: boolean;
	strings: boolean;
}

interface IFileType {
	readonly name: FileLang;
	readonly hlOptions: HighlightingOptions;
	get flags(): HighlightingOptions;
}

/**
 * The base class for File Types
 */
export abstract class AbstractFileType implements IFileType {
	public readonly name: FileLang = FileLang.Plain;
	public readonly hlOptions: HighlightingOptions = {
		numbers: false,
		strings: false,
	};

	get flags(): HighlightingOptions {
		return this.hlOptions;
	}
}

// ----------------------------------------------------------------------------
// FileType implementations
// ----------------------------------------------------------------------------
const defaultHighlightOptions: HighlightingOptions = {
	numbers: true,
	strings: true,
};

class TypeScriptFile extends AbstractFileType {
	public readonly name: FileLang = FileLang.TypeScript;
	public readonly hlOptions: HighlightingOptions = {
		...defaultHighlightOptions,
	};
}

class JavaScriptFile extends AbstractFileType {
	public readonly name: FileLang = FileLang.JavaScript;
	public readonly hlOptions: HighlightingOptions = {
		...defaultHighlightOptions,
	};
}

class CSSFile extends AbstractFileType {
	public readonly name: FileLang = FileLang.CSS;
	public readonly hlOptions: HighlightingOptions = {
		...defaultHighlightOptions,
	};
}

// ----------------------------------------------------------------------------
// External interface
// ----------------------------------------------------------------------------

export class FileType extends AbstractFileType {
	static #fileTypeMap = new Map([
		['.css', CSSFile],
		['.js', JavaScriptFile],
		['.jsx', JavaScriptFile],
		['.mjs', JavaScriptFile],
		['.ts', TypeScriptFile],
		['.tsx', TypeScriptFile],
	]);

	public static default(): FileType {
		return new FileType();
	}

	public static from(filename: string): FileType {
		const ext = path.extname(filename);
		const type = FileType.#fileTypeMap.get(ext) ?? FileType;

		return new type();
	}
}
